/*-----------------------------------------------------------------
 File: adder.sv
 
 Adds 5 to input value
-----------------------------------------------------------------*/

`timescale 1ps / 1ps
`include "axis_if.vh"


module adder(
    input  logic      ACLK,
    input  logic      ARESETN,
    
    axis.axis_mst_mp  AXIS_MST,     // AXI stream interface, master modport
    axis.axis_slv_mp  AXIS_SLV      // AXI stream interface, slave modport   
    
    );


   // control state machine coding
  enum {Init, Idle, Add, Oput} currState, nextState;

   // internal signals
  logic [15:0] tdata_reg;
  


  /*------------------------------------------------------------------
     Control state machine.
   ------------------------------------------------------------------*/
    
  // current state registers
  always_ff @(posedge ACLK) begin: setCurrState
  
    if (ARESETN == 0) begin
      currState <= Init;    
    end
    else begin
      currState <= nextState;    
    end 
  
  end: setCurrState
  
  

  // next state
  always_comb begin : setNextState
    nextState = currState;      // default is to stay in current state
        
    unique case (currState)
     Init :                           nextState = Idle;
     Idle : if (AXIS_SLV.tvalid == 1) nextState = Add;
     Add :                            nextState = Oput;
     Oput : if (AXIS_MST.tready == 1) nextState = Idle;
    endcase

  end : setNextState
  
  
  
 /*------------------------------------------------------------------
    Capture input data from TDATA Slave
  ------------------------------------------------------------------*/  
   always_ff @(posedge ACLK) begin
   
     if (ARESETN == 0) begin
       tdata_reg <= 0;
     end
     else if (AXIS_SLV.tready == 1 && AXIS_SLV.tvalid == 1) begin
       tdata_reg <= AXIS_SLV.tdata;
     end
     
   end
  
  

 /*------------------------------------------------------------------
    TREADY output for AXI-Stream Slave Interface.
    Indicates that AXI-Stream Slave interface is ready to accept
    new data. 
    Resets to low, then set high when exiting Init state.
    Will stay high until exiting Idle state. Goes high again    
    when re-entering Idle state.
  ------------------------------------------------------------------*/

  always_ff @(posedge ACLK) begin : setTREADY_Slave
   
     if (ARESETN == 0) begin
       AXIS_SLV.tready <= 0;
     end
     else if
         ( (currState == Init) ||                                 // Init -> Idle
  		   (currState == Idle && AXIS_SLV.tvalid == 0 ) ||          // looping in Idle state
		     (currState == Oput && AXIS_MST.tready == 1 ) ) begin     // Oput -> Idle
         AXIS_SLV.tready <= 1;
       end
       else begin
         AXIS_SLV.tready <= 0;
       end
     
   end : setTREADY_Slave


  /*------------------------------------------------------------------
    TVALID output for AXI-Stream Master Interface.
    Indicates that encrypted data is ready to be read from 
    AXI-Stream Master interface.
    Resets to low, goes to high when entering Oput state.
    Goes low again exiting Oput state.
  ------------------------------------------------------------------*/
  
  always_ff @(posedge ACLK) begin : setTVALID_Master
  
     if (ARESETN == 0) begin
       AXIS_MST.tvalid <= 0;
     end
     else if ((currState == Add) || (currState == Oput && AXIS_MST.tready == 0)) begin
       AXIS_MST.tvalid <= 1;
     end
     else begin
       AXIS_MST.tvalid <= 0;
     end
     
  end : setTVALID_Master

  
  /*------------------------------------------------------------------
    TDATA output for AXI-Stream Master Interface.
  ------------------------------------------------------------------*/
   
  always_ff @(posedge ACLK) begin : setTDATA_Master
  
     if (ARESETN == 0) begin
       AXIS_MST.tdata <= 0;
     end
     else if (currState == Add) begin
       AXIS_MST.tdata <= tdata_reg + 5;
     end
     
  end : setTDATA_Master   
   
   
   
  
endmodule : adder
/*-----------------------------------------------------------------
-- End of File: adder.sv
-----------------------------------------------------------------*/


